'use strict';

var path = require('path');
var through = require('through2');
var PluginError = require('plugin-error');
var jimp = require('jimp');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var through__default = /*#__PURE__*/_interopDefaultLegacy(through);
var PluginError__default = /*#__PURE__*/_interopDefaultLegacy(PluginError);
var jimp__default = /*#__PURE__*/_interopDefaultLegacy(jimp);

const PLUGIN_NAME = 'gulp-jimp-wrapper';
const supportedMimetypes = {
  jpeg: 'image/jpeg',
  jpg: 'image/jpeg',
  png: 'image/png',
  bmp: 'image/bmp',
  tiff: 'image/tiff',
  gif: 'image/gif'
};

const processImg = (img, cb) =>
  new Promise((resolve, reject) => {
    const extension = path__default['default']
      .extname(img.path)
      .split('.')
      .pop();

    jimp__default['default']
      .read(img.contents)
      .then(cb)
      .then(img => img.getBufferAsync(supportedMimetypes[extension]))
      .then(data => resolve(data))
      .catch(err => reject(err));
  });

const jimpWrapper = cb => {
  return through__default['default'].obj(async function (img, _, callback) {
    if (img.isNull()) {
      callback(null, img);
      return
    }

    if (img.isStream()) {
      callback(new PluginError__default['default'](PLUGIN_NAME, 'Streaming not supported'));
      return
    }

    if (typeof cb !== 'function') {
      callback(
        new PluginError__default['default'](PLUGIN_NAME, `Argument \`${cb}\` is not a function`)
      );
      return
    }

    try {
      const data = await processImg(img, cb);
      img.contents = Buffer.from(data);
      this.push(img);
    } catch (err) {
      this.emit('error', new PluginError__default['default'](PLUGIN_NAME, err));
    }

    callback();
  })
};

var src = jimpWrapper;

module.exports = src;
